#!/usr/local/bin/python3
"""
All commands go through this script
Purpose: Collecting user data
"""


import os

import json
from sys import argv
import requests

# Asciinema things
import asciinema
import asciinema.config as config
from asciinema.api import Api
from asciinema.commands.record import RecordCommand


def main():

    # Load default asciinema configuration
    cfg = config.load()
    # args = {}
    api = Api(cfg.api_url, os.environ.get("USER"), cfg.api_token)
    # args.command = None,
    # args.filename = 'roa',
    # args.max_wait = None,
    # args.quiet = False,
    # args.title = None,
    # args.yes = False
    RecordCommand(api, 'roa', None, None, False, False, None).execute()

    """
    Get command, args, exit code
    Send those data to server for business logic
    Returns:
        Not return anything, print result on terminal
    Raises:
        all: do nothing
    """

    # exit_code = argv[1]
    # command = argv[2]
    # arguments_list = argv[3:]

    # print "-------debug------------"
    # print ( command )
    # print ( exit_code )
    # print ( ' '.join(arguments_list) )
    # asciinema.recorder.exe

    # args = ' '.join(arguments_list)

    # Terminal tracking
    # url = "http://localhost:5000"
    # data = {
    #     'command': command,
    #     'exit_code': exit_code,
    #     'args': args
    # }
    # headers = {'content-type': 'application/json'}
    # try:
    #     r = requests.post(url=url, data=json.dumps(data), headers=headers)
    #     r_stat = r.text
    #     # print r.status_code
    #     print (r_stat)
    # except:
    #     # do nothing
    #     pass


if __name__ == "__main__":
    main()



import platform
import re

from asciinema import __version__
from asciinema.urllib_http_adapter import URLLibHttpAdapter
from asciinema.http_adapter import HTTPConnectionError


class APIError(Exception):
    pass


class Api:

    def __init__(self, url, user, token, http_adapter=None):
        self.url = url
        self.user = user
        self.token = token
        self.http_adapter = http_adapter if http_adapter is not None else URLLibHttpAdapter()

    def auth_url(self):
        return "{}/connect/{}".format(self.url, self.token)

    def upload_url(self):
        return "{}/api/asciicasts".format(self.url)

    def upload_asciicast(self, path):
        with open(path, 'rb') as f:
            try:
                status, headers, body = self.http_adapter.post(
                    self.upload_url(),
                    files={"asciicast": ("asciicast.json", f)},
                    headers=self._headers(),
                    username=self.user,
                    password=self.token
                )
            except HTTPConnectionError as e:
                raise APIError(str(e))

        if status != 200 and status != 201:
            self._handle_error(status, body)

        return body, headers.get('Warning')

    def _headers(self):
        return {'User-Agent': self._user_agent()}

    def _user_agent(self):
        os = re.sub('([^-]+)-(.*)', '\\1/\\2', platform.platform())

        return 'asciinema/%s %s/%s %s' % (__version__,
                                          platform.python_implementation(),
                                          platform.python_version(),
                                          os
                                          )

    def _handle_error(self, status, body):
        errors = {
            400: "Invalid request: %s" % body,
            401: "Invalid or revoked recorder token",
            404: "API endpoint not found. This asciinema version may no longer be supported. Please upgrade to the latest version.",
            413: "Sorry, your asciicast is too big.",
            422: "Invalid asciicast: %s" % body,
            503: "The server is down for maintenance. Try again in a minute."
        }

        error = errors.get(status)

        if not error:
            if status >= 500:
                error = "The server is having temporary problems. Try again in a minute."
            else:
                error = "HTTP status: %i" % status

        raise APIError(error)



"""
Author: Trieu Pham (trieu@codelynx.io)

A private property of #codelynx company
"""

import json
import sys
from asciinema.commands.command import Command

import requests


class Api(Command):
    """
    This class manages talking with api server
    """

    def __init__(self):
        try:
            with open('./conf') as f:
                self.url = f.readline().strip()
        except Exception as ex:
            with open('./conf', '+w') as f:
                f.write("http://localhost:5000")
            self.url = "http://localhost:5000"
        # self.url = "http://localhost:5000"
        Command.__init__(self)  # color

    def handle_data(self, collection, quiet=True, method='POST', **kwargs):
        """
            Send char/output to server
            @params:
                collection: insert into this collection <str>
                quiet: display result on screen or not <bool>
                **kwargs: tuple of data <str>
            @returns: text
            @raises: exit program
        """
        data = {'collection': collection, 'kwargs': kwargs}
        headers = {'content-type': 'application/json'}
        try:
            if method == 'POST':
                result = requests.post(
                    url=self.url,
                    data=json.dumps(data),
                    headers=headers)
            if method == 'GET':
                result = requests.get(
                    url=self.url + '/lookup/get',
                    params={"question": kwargs['question']}
                )
            # print(lookups.url)
            # print(result.text)
            if not quiet:  # Print return info if required
                if result.status_code == 404:
                    self.print_warning(result.text)
                elif result.status_code == 500:
                    self.print_error(result.text)
                else:
                    self.print_info(result.text)
                # if 'Success' in result.text:
                #     self.print_info(result.text)
                # if 'False' in result.text:
                #     self.print_warning(result.text)
            # return r.status_code
            # return r.text
            return result
        except requests.exceptions.ConnectionError as e:
            # Print info in red and exit whole program
            sys.exit('\x1b[31m\n~ Cannot connect server, exit now...\x1b[0m')



from asciinema.commands.command import Command


class AuthCommand(Command):

    def __init__(self, api_url, api_token):
        Command.__init__(self)
        self.api_url = api_url
        self.api_token = api_token

    def execute(self):
        url = '%s/connect/%s' % (self.api_url, self.api_token)
        self.print('Open the following URL in a browser to register your API '
                   'token and assign any recorded asciicasts to your profile:\n'
                   '%s' % url)



"""
Author: Trieu Pham (trieu@codelynx.io)

A private property of #codelynx company
"""

import sys
from codelynx.api import Api
from asciinema.commands.command import Command


class Ask(Command):
    """
    This class manages asking xirion a question
    Expected command: xirion ask <question>
    Response: print xirion answer
    @args:
        question: ask xirion this question
    @returns:
        0 if everything 's good, 1 otherwise
    @raises:
        unknown
    """

    def __init__(self, question):
        # self.cmd = cmd
        # self.correction = correction
        self.api = Api()
        self.question = question
        Command.__init__(self)  # color

    def execute(self):
        """
        Execute asking
        """
        try:
             # Debug
            # print(self.question)
            # lookup quetions here
            # self.api.handle_data('lookup', quiet=False, orig=self.question)
            answer = self.api.handle_data('lookup', quiet=False, method='GET', question=self.question)
        except Exception as ex:
            print(ex)
            return 1
        return 0



import os
import subprocess

from .asciicast import Asciicast
from .pty_recorder import PtyRecorder
from .stdout import Stdout


class Recorder:

    def __init__(self, pty_recorder=None, env=None):
        self.pty_recorder = pty_recorder if pty_recorder is not None else PtyRecorder()
        self.env = env if env is not None else os.environ

    def record(self, path, user_command, title, max_wait):
        command = user_command or self.env.get('SHELL') or 'sh'
        stdout = Stdout(max_wait)
        env = os.environ.copy()
        env['ASCIINEMA_REC'] = '1'

        self.pty_recorder.record_command(['sh', '-c', command], stdout, env)

        width = int(subprocess.check_output(['tput', 'cols']))
        height = int(subprocess.check_output(['tput', 'lines']))

        asciicast = Asciicast(
            stdout,
            width,
            height,
            stdout.duration,
            command=user_command,
            title=title,
            term=self.env.get('TERM'),
            shell=self.env.get('SHELL')
        )

        asciicast.save(path)



import os
import os.path as path
import sys
import uuid
import configparser


class ConfigError(Exception):
    pass


DEFAULT_API_URL = 'https://asciinema.org'


class Config:

    def __init__(self, config, env=None):
        self.config = config
        self.env = env if env is not None else os.environ

    @property
    def api_url(self):
        return self.env.get(
            'ASCIINEMA_API_URL',
            self.config.get('api', 'url', fallback=DEFAULT_API_URL)
        )

    @property
    def api_token(self):
        try:
            return self.env.get('ASCIINEMA_API_TOKEN') or self.config.get('api', 'token')
        except (configparser.NoOptionError, configparser.NoSectionError):
            try:
                return self.config.get('user', 'token')
            except (configparser.NoOptionError, configparser.NoSectionError):
                raise ConfigError('no API token found in config file, and ASCIINEMA_API_TOKEN is unset')

    @property
    def record_command(self):
        return self.config.get('record', 'command', fallback=None)

    @property
    def record_max_wait(self):
        return self.config.getfloat('record', 'maxwait', fallback=None)

    @property
    def record_yes(self):
        return self.config.getboolean('record', 'yes', fallback=False)

    @property
    def record_quiet(self):
        return self.config.getboolean('record', 'quiet', fallback=False)

    @property
    def play_max_wait(self):
        return self.config.getfloat('play', 'maxwait', fallback=None)

    @property
    def play_speed(self):
        return self.config.getfloat('play', 'speed', fallback=1.0)


def load_file(paths):
    config = configparser.ConfigParser()
    read_paths = config.read(paths)

    if read_paths:
        return config


def create_file(filename):
    config = configparser.ConfigParser()
    config['api'] = {}
    config['api']['token'] = str(uuid.uuid4())

    if not path.exists(path.dirname(filename)):
        os.makedirs(path.dirname(filename))

    with open(filename, 'w') as f:
        config.write(f)

    return config


def load(env=os.environ):
    paths = []

    asciinema_config_home = env.get("ASCIINEMA_CONFIG_HOME")
    xdg_config_home = env.get("XDG_CONFIG_HOME")
    home = env.get("HOME")

    if asciinema_config_home:
        paths.append(path.join(asciinema_config_home, "config"))
    elif xdg_config_home:
        paths.append(path.join(xdg_config_home, "asciinema", "config"))
    elif home:
        paths.append(path.join(home, ".asciinema", "config"))
        paths.append(path.join(home, ".config", "asciinema", "config"))
    else:
        raise Exception("need $ASCIINEMA_CONFIG_HOME or $XDG_CONFIG_HOME or $HOME")

    config = load_file(paths) or create_file(paths[-1])

    return Config(config, env)



import tty
import select
import os


class raw():
    def __init__(self, fd):
        self.fd = fd
        self.restore = False

    def __enter__(self):
        try:
            self.mode = tty.tcgetattr(self.fd)
            tty.setraw(self.fd)
            self.restore = True
        except tty.error:  # This is the same as termios.error
            pass

    def __exit__(self, type, value, traceback):
        if self.restore:
            tty.tcsetattr(self.fd, tty.TCSAFLUSH, self.mode)


def read_non_blocking(fd):
    data = b''

    while fd in select.select([fd], [], [], 0)[0]:
        data += os.read(fd, 1024)

    return data



import sys
import asciinema
import codelynx
from setuptools import setup

if sys.version_info[0] < 3:
    sys.exit('Python < 3 is unsupported.')

# url_template = 'https://github.com/asciinema/asciinema/archive/v%s.tar.gz'
requirements = []

setup(
    name='xirion',
    version=codelynx.__version__,
    packages=[
        'asciinema', 'asciinema.commands',
        'codelynx', 'codelynx.commands'
    ],
    license='GNU GPLv3',
    description='Terminal session recorder',
    author=codelynx.__author__,
    author_email='trieu@codelynx.io',
    url='http://codelynx.io',
    # download_url=(url_template % asciinema.__version__),
    entry_points={
        'console_scripts': [
            # 'codelynx = codelynx.__main__:main',
            'xirion = asciinema.__main__:main',
        ],
    },
    install_requires=requirements,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: System :: Shells',
        'Topic :: Terminals',
        'Topic :: Utilities'
    ],
)



import time
import codecs


class Stdout:

    def __init__(self, max_wait=None):
        self.frames = []
        self.max_wait = max_wait
        self.last_write_time = time.time()
        self.duration = 0
        self.decoder = codecs.getincrementaldecoder('UTF-8')('replace')

    def write(self, data):
        # print('stdout write: %s' % data.decode('utf-8'))
        # text = self.decoder.decode(data)
        text = str(data, 'utf-8')
        if text:
            delay = self._increment_elapsed_time()
            self.frames.append([delay, text])

        return len(data)

    def close(self):
        self._increment_elapsed_time()

        if len(self.frames) > 0:
            last_frame = self.frames[-1]
            if last_frame[1] == "exit\r\n" or last_frame[1] == "logout\r\n":
                self.frames = self.frames[0:-1]
                self.duration -= last_frame[0]

    def _increment_elapsed_time(self):
        # delay = int(delay * 1000000) / 1000000.0 # millisecond precission
        now = time.time()
        delay = now - self.last_write_time

        if self.max_wait and delay > self.max_wait:
            delay = self.max_wait

        self.duration += delay
        self.last_write_time = now

        return delay



from asciinema.commands.command import Command
from asciinema.api import APIError


class UploadCommand(Command):

    def __init__(self, api, filename):
        Command.__init__(self)
        self.api = api
        self.filename = filename

    def execute(self):
        try:
            url, warn = self.api.upload_asciicast(self.filename)

            if warn:
                self.print_warning(warn)

            self.print(url)

        except FileNotFoundError as e:
            self.print_warning("Upload failed: %s" % str(e))
            return 1

        except APIError as e:
            self.print_warning("Upload failed: %s" % str(e))
            self.print_warning("Retry later by running: asciinema upload %s" % self.filename)
            return 1

        return 0



class Command:

    def __init__(self, quiet=False):
        self.quiet = quiet

    def print(self, text):
        if not self.quiet:
            print(text)

    def print_info(self, text):
        if not self.quiet:
            print("\x1b[32m~ %s\x1b[0m" % text)

    def print_warning(self, text):
        if not self.quiet:
            print("\x1b[33m~ %s\x1b[0m" % text)

    def print_error(self, text):
        if not self.quiet:
            print("\x1b[31m~ %s\x1b[0m" % text)
